@@ -14,9 +14,9 @@
 
 "Transport layer"
 
-from session import Session, SessionListener
-from ssh import SSHSession
-from errors import *
+from ncclient.transport.session import Session, SessionListener, NetconfBase
+from ncclient.transport.ssh import SSHSession
+from ncclient.transport.errors import *
 
 __all__ = [
     'Session',
@@ -25,6 +25,7 @@ __all__ = [
     'TransportError',
     'AuthenticationError',
     'SessionCloseError',
+    'NetconfBase',
     'SSHError',
     'SSHUnknownHostError'
 
